#include <libpv_config.h>

#include <errno.h>
#include <string.h>

#include <libpv.h>
#include <pv_register.h>


static const char * const pv_type_str[] = {
	[PV_TYPE_INT8]		= "int8_t",
	[PV_TYPE_UINT8]		= "uint8_t",

	[PV_TYPE_INT16]		= "int16_t",
	[PV_TYPE_UINT16]	= "uint16_t",

	[PV_TYPE_INT32]		= "int32_t",
	[PV_TYPE_UINT32]	= "uint32_t",

	[PV_TYPE_INT64]		= "int64_t",
	[PV_TYPE_UINT64]	= "uint64_t",

	[PV_TYPE_STRING]	= "string",

	[PV_TYPE_FLOAT]		= "float",

	[PV_TYPE_DOUBLE]	= "double",

	[PV_TYPE_INT8_ARRAY]	= "int8_t[]",
	[PV_TYPE_UINT8_ARRAY]	= "uint8_t[]",

	[PV_TYPE_INT16_ARRAY]	= "int16_t[]",
	[PV_TYPE_UINT16_ARRAY]	= "uint16_t[]",

	[PV_TYPE_INT32_ARRAY]	= "int32_t[]",
	[PV_TYPE_UINT32_ARRAY]	= "uint32_t[]",

	[PV_TYPE_FLOAT_ARRAY]	= "float[]",

	[PV_TYPE_DOUBLE_ARRAY]	= "double[]",
};

/**
 * pv_register_lookup
 * @pv_engine: Engine to work with
 * @id: id value defined in <pv> tag
 *
 * Get the pointer to a process resource with ID equal to @id
 *
 * Return value: Pointer to struct pv or NULL if this <pv> @id
 * doesn't exist
 **/
struct pv
*pv_register_lookup(struct pv_engine	*pv_engine,
		    const char		*id)
{
	struct pv		*pv;
	int			finished = 0;

	for (goto_head(&pv_engine->pv_list); !finished; goto_next(&pv_engine->pv_list)) {
		pv = (struct pv *)list_get_element_data(pv_engine->pv_list);

		if (is_last(pv_engine->pv_list))
			finished = 1;

		if (!strcmp(id, pv->id)) {
			return pv;
		}
	}

	dprintf(LEVEL_DEBUG, "(%s) variable with id \"%s\" not found\n",
		__PRETTY_FUNCTION__, id);

	errno = ENXIO;
	return NULL;
}


static struct pv
*pv_register_lookup_check(struct pv_engine	*pv_engine,
			  const char		*id,
			  enum pv_type		type)
{
	struct pv	*pv;

	pv = pv_register_lookup(pv_engine, id);
	if (!pv)
		return NULL;

	dprintf(LEVEL_DEBUG, "(%s) variable \"%s\" requested type %s found type is %s\n",
		__PRETTY_FUNCTION__,
		id,
		pv_type_str[type],
		pv_type_str[pv->type]);

	if (pv->type != type) {
		errno = ENOTSUP;
		return NULL;
	}

	return pv;
}

/**
 * pv_register_uint8
 * @pv_engine_anon: Engine to work with
 * @id: id value defined in <pv> tag
 *
 * Get the pointer to the process resource of 8 bit
 * width and unsigned int type with ID equal to @id
 *
 * Return value: Pointer to unint8_t or NULL if this <pv> @id
 * doesn't exist
 **/
uint8_t *pv_register_uint8(pv_engine_t	*pv_engine_anon,
			   const char	*id)
{
	struct pv_engine	*pv_engine = (struct pv_engine *)pv_engine_anon;
	struct pv		*pv;

	if (!pv_engine || !id)
		return NULL;

	pv = pv_register_lookup_check(pv_engine, id, PV_TYPE_UINT8);
	if (!pv)
		return NULL;

	return pv_to_abs_addr(pv_engine, pv)->u8;
}

/**
 * pv_register_int8
 * @pv_engine_anon: Engine to work with
 * @id: id value defined in <pv> tag
 *
 * Get the pointer to the process resource of 8 bit
 * width and signed int type with ID equal to @id
 *
 * Return value: Pointer to int8_t or NULL if this <pv> @id
 * doesn't exist
 **/
int8_t *pv_register_int8(pv_engine_t	*pv_engine_anon,
			 const char	*id)
{
	struct pv_engine	*pv_engine = (struct pv_engine *)pv_engine_anon;
	struct pv		*pv;

	if (!pv_engine || !id)
		return NULL;

	pv = pv_register_lookup_check(pv_engine, id, PV_TYPE_INT8);
	if (!pv)
		return NULL;

	return pv_to_abs_addr(pv_engine, pv)->s8;
}

/**
 * pv_register_uint16
 * @pv_engine_anon: Engine to work with
 * @id: id value defined in <pv> tag
 *
 * Get the pointer to the process resource of 16 bit
 * width and unsigned int type with ID equal to @id
 *
 * Return value: Pointer to uint16_t or NULL if this <pv> @id
 * doesn't exist
 **/
uint16_t *pv_register_uint16(pv_engine_t	*pv_engine_anon,
			     const char		*id)
{
	struct pv_engine	*pv_engine = (struct pv_engine *)pv_engine_anon;
	struct pv		*pv;

	if (!pv_engine || !id)
		return NULL;

	pv = pv_register_lookup_check(pv_engine, id, PV_TYPE_UINT16);
	if (!pv)
		return NULL;

	return pv_to_abs_addr(pv_engine, pv)->u16;
}

/**
 * pv_register_int16
 * @pv_engine_anon: Engine to work with
 * @id: id value defined in <pv> tag
 *
 * Get the pointer to the process resource of 16 bit
 * width and signed int type with ID equal to @id
 *
 * Return value: Pointer to int16_t or NULL if this <pv> @id
 * doesn't exist
 **/
int16_t *pv_register_int16(pv_engine_t	*pv_engine_anon,
			   const char	*id)
{
	struct pv_engine	*pv_engine = (struct pv_engine *)pv_engine_anon;
	struct pv		*pv;

	if (!pv_engine || !id)
		return NULL;

	pv = pv_register_lookup_check(pv_engine, id, PV_TYPE_INT16);
	if (!pv)
		return NULL;

	return pv_to_abs_addr(pv_engine, pv)->s16;
}

/**
 * pv_register_uint32
 * @pv_engine_anon: Engine to work with
 * @id: id value defined in <pv> tag
 *
 * Get the pointer to the process resource of 32 bit
 * width and unsigned int type with ID equal to @id
 *
 * Return value: Pointer to uint32_t or NULL if this <pv> @id
 * doesn't exist
 **/
uint32_t *pv_register_uint32(pv_engine_t	*pv_engine_anon,
			     const char		*id)
{
	struct pv_engine	*pv_engine = (struct pv_engine *)pv_engine_anon;
	struct pv		*pv;

	if (!pv_engine || !id)
		return NULL;

	pv = pv_register_lookup_check(pv_engine, id, PV_TYPE_UINT32);
	if (!pv)
		return NULL;

	return pv_to_abs_addr(pv_engine, pv)->u32;
}

/**
 * pv_register_int32
 * @pv_engine_anon: Engine to work with
 * @id: id value defined in <pv> tag
 *
 * Get the pointer to the process resource of 32 bit
 * width and signed int type with ID equal to @id
 *
 * Return value: Pointer to int32_t or NULL if this <pv> @id
 * doesn't exist
 **/
int32_t *pv_register_int32(pv_engine_t	*pv_engine_anon,
			   const char	*id)
{
	struct pv_engine	*pv_engine = (struct pv_engine *)pv_engine_anon;
	struct pv		*pv;

	if (!pv_engine || !id)
		return NULL;

	pv = pv_register_lookup_check(pv_engine, id, PV_TYPE_INT32);
	if (!pv)
		return NULL;

	return pv_to_abs_addr(pv_engine, pv)->s32;
}

float *pv_register_float(pv_engine_t	*pv_engine_anon,
			 const char	*id)
{
	struct pv_engine	*pv_engine = (struct pv_engine *)pv_engine_anon;
	struct pv		*pv;

	if (!pv_engine || !id)
		return NULL;

	pv = pv_register_lookup_check(pv_engine, id, PV_TYPE_FLOAT);
	if (!pv)
		return NULL;

	return pv_to_abs_addr(pv_engine, pv)->f;
}

double *pv_register_double(pv_engine_t	*pv_engine_anon,
			   const char	*id)
{
	struct pv_engine	*pv_engine = (struct pv_engine *)pv_engine_anon;
	struct pv		*pv;

	if (!pv_engine || !id)
		return NULL;

	pv = pv_register_lookup_check(pv_engine, id, PV_TYPE_DOUBLE);
	if (!pv)
		return NULL;

	return pv_to_abs_addr(pv_engine, pv)->d;
}

char *pv_register_string(pv_engine_t	*pv_engine_anon,
			 const char	*id)
{
	struct pv_engine	*pv_engine = (struct pv_engine *)pv_engine_anon;
	struct pv		*pv;

	if (!pv_engine || !id)
		return NULL;

	pv = pv_register_lookup_check(pv_engine, id, PV_TYPE_STRING);
	if (!pv)
		return NULL;

	return pv_to_abs_addr(pv_engine, pv)->string;
}


uint8_t *pv_register_uint8_array(pv_engine_t	*pv_engine_anon,
				 const char	*id)
{
	struct pv_engine	*pv_engine = (struct pv_engine *)pv_engine_anon;
	struct pv		*pv;

	if (!pv_engine || !id)
		return NULL;

	pv = pv_register_lookup_check(pv_engine, id, PV_TYPE_UINT8_ARRAY);
	if (!pv)
		return NULL;

	return pv_to_abs_addr(pv_engine, pv)->u8;
}


int8_t *pv_register_int8_array(pv_engine_t	*pv_engine_anon,
			       const char	*id)
{
	struct pv_engine	*pv_engine = (struct pv_engine *)pv_engine_anon;
	struct pv		*pv;

	if (!pv_engine || !id)
		return NULL;

	pv = pv_register_lookup_check(pv_engine, id, PV_TYPE_INT8_ARRAY);
	if (!pv)
		return NULL;

	return pv_to_abs_addr(pv_engine, pv)->s8;
}

uint16_t *pv_register_uint16_array(pv_engine_t	*pv_engine_anon,
				   const char	*id)
{
	struct pv_engine	*pv_engine = (struct pv_engine *)pv_engine_anon;
	struct pv		*pv;

	if (!pv_engine || !id)
		return NULL;

	pv = pv_register_lookup_check(pv_engine, id, PV_TYPE_UINT16_ARRAY);
	if (!pv)
		return NULL;

	return pv_to_abs_addr(pv_engine, pv)->u16;
}

int16_t *pv_register_int16_array(pv_engine_t	*pv_engine_anon,
				 const char	*id)
{
	struct pv_engine	*pv_engine = (struct pv_engine *)pv_engine_anon;
	struct pv		*pv;

	if (!pv_engine || !id)
		return NULL;

	pv = pv_register_lookup_check(pv_engine, id, PV_TYPE_INT16_ARRAY);
	if (!pv)
		return NULL;

	return pv_to_abs_addr(pv_engine, pv)->s16;
}

uint32_t *pv_register_uint32_array(pv_engine_t	*pv_engine_anon,
				   const char	*id)
{
	struct pv_engine	*pv_engine = (struct pv_engine *)pv_engine_anon;
	struct pv		*pv;

	if (!pv_engine || !id)
		return NULL;

	pv = pv_register_lookup_check(pv_engine, id, PV_TYPE_UINT32_ARRAY);
	if (!pv)
		return NULL;

	return pv_to_abs_addr(pv_engine, pv)->u32;
}

int32_t *pv_register_int32_array(pv_engine_t	*pv_engine_anon,
				 const char	*id)
{
	struct pv_engine	*pv_engine = (struct pv_engine *)pv_engine_anon;
	struct pv		*pv;

	if (!pv_engine || !id)
		return NULL;

	pv = pv_register_lookup_check(pv_engine, id, PV_TYPE_INT32_ARRAY);
	if (!pv)
		return NULL;

	return pv_to_abs_addr(pv_engine, pv)->s32;
}

float *pv_register_float_array(pv_engine_t	*pv_engine_anon,
			       const char	*id)
{
	struct pv_engine	*pv_engine = (struct pv_engine *)pv_engine_anon;
	struct pv		*pv;

	if (!pv_engine || !id)
		return NULL;

	pv = pv_register_lookup_check(pv_engine, id, PV_TYPE_FLOAT_ARRAY);
	if (!pv)
		return NULL;

	return pv_to_abs_addr(pv_engine, pv)->f;
}

double *pv_register_double_array(pv_engine_t	*pv_engine_anon,
				 const char	*id)
{
	struct pv_engine	*pv_engine = (struct pv_engine *)pv_engine_anon;
	struct pv		*pv;

	if (!pv_engine || !id)
		return NULL;

	pv = pv_register_lookup_check(pv_engine, id, PV_TYPE_DOUBLE_ARRAY);
	if (!pv)
		return NULL;

	return pv_to_abs_addr(pv_engine, pv)->d;
}

